"
 COPYRIGHT (c) 1997 by Claus Gittinger / eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"




Object subclass:#DataSetColumn
	instanceVariableNames:'columnNumber dataSet label minWidth width description form form2
		formInset textInset writeSelector drawableAction backgroundColor
		foregroundColor fgSelector bgSelector'
	classVariableNames:''
	poolDictionaries:''
	category:'Views-DataSet'
!

!DataSetColumn class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1997 by Claus Gittinger / eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"



!

documentation
"
    represent one single column description of a DataSetView

    [Instance variables:]

        description     <DataSetColumnSpec>     the column description
        dataSet         <DSVColumnView>         the view it belongs to
        width           <Integer>               width of column
        minWidth        <Integer>               minimum required width by the column
        formInset       <Integer>               top inset of a form
        textInset       <Integer>               top inset of a string
        form            <Form>                  a form drawn when a cell is unselected
        form2           <Form>                  a second form (used by Toggle off).
        label           <misc>                  cellLabel on device

    [author:]
        Claus Atzkern

    [see also:]
        DataSetColumnSpec
        DSVColumnView
        DataSetView
"


! !

!DataSetColumn methodsFor:'accessing'!

at:aRowNr
    "get the value of the raw at an index, aRowNr
    "
    |row|

    row := dataSet at:aRowNr.
    ^ description row:row at:columnNumber
!

at:aRowNr put:something
    "set the value of the raw at an index, aRowNr
    "
    description row:(dataSet at:aRowNr) at:columnNumber put:something
!

backgroundColor
    ^ backgroundColor
!

foregroundColor
    ^ foregroundColor
!

label
    "get the label assigned to the column
    "
    ^ label


!

width
    "returns the width in pixels
    "
    |end|

    width notNil ifTrue:[ ^ width ].                            "/ already computed

    (    (width := description width)    ~~ 0                   "/ fixed size
     or:[(width := description minWidth) ~~ 0]                  "/ start size
    ) ifFalse:[
        form2 isNil ifTrue:[                                    "/ has drawable labels
            "/ take maximum 10 entries to calculate the width;
            "/ can be resized later if neccessary

            end := 10 min:dataSet size.

            1 to:end do:[:aRowNr||lbl|
                lbl := self drawableAt:aRowNr.

                (lbl respondsTo:#widthOn:) ifTrue:[
                    width := width max:(lbl widthOn:dataSet)
                ] ifFalse:[
                    lbl notNil ifTrue:[
                        width := width max:(lbl displayString widthOn:dataSet)
                    ]
                ]
            ].
            description editorType ~~ #None ifTrue:[
                width := width + (dataSet font widthOn:dataSet device)
            ].
            form notNil ifTrue:[
                width := width + form width
            ]
        ] ifFalse:[
            width := width + form2 width
        ].
        label notNil ifTrue:[
            width := width max:(label widthOn:dataSet)
        ].
    ].
    ^ width := minWidth := width + dataSet separatorSize + (2 * dataSet horizontalSpacing).
!

width:aWidth
    "set the width
    "
    |w|

    (w := description width) ~~ 0 ifTrue:[             "/ fixed size
        width := minWidth := w
    ] ifFalse:[
        width := aWidth max:minWidth
    ].

! !

!DataSetColumn methodsFor:'drawing'!

drawFrom:start to:stop yTop:yTop yBot:yBot with:fgColor and:bgColor
    "redraw rows between start and stop
    "
    |lgCol dkCol fg bg drawColSep n isLast lblFg lblBg innerH
     rowHeight "{ Class:SmallInteger }"
     nTimes    "{ Class:SmallInteger }"
     sepSize   "{ Class:SmallInteger }"
     y         "{ Class:SmallInteger }"
     y1        "{ Class:SmallInteger }"
     x         "{ Class:SmallInteger }"
     w         "{ Class:SmallInteger }"
    |
    rowHeight := dataSet rowHeight.
    nTimes    := stop - start + 1.
    sepSize   := dataSet separatorSize.

    (bg := backgroundColor) isNil ifTrue:[
        bg := bgColor
    ] ifFalse:[
        dataSet paint:bg.
        dataSet fillRectangleX:0 y:yTop width:width height:(nTimes * rowHeight - 1)
    ].
    (fg := foregroundColor) notNil ifTrue:[
        lgCol := fg
    ] ifFalse:[
        fg := fgColor.
        lgCol := dataSet hgLgFgColor.
    ].
    dkCol  := dataSet hgLgBgColor.
    y      := yTop.
    x      := dataSet horizontalSpacing.
    innerH := rowHeight - 1 - sepSize.

    start to:stop do:[:i||row lbl|
        row := dataSet at:i.

        (dataSet isSelected:i inColumn:columnNumber) ifTrue:[
            dataSet paint:dkCol.
            dataSet fillRectangleX:0 y:y width:width height:innerH.
            lblFg := lgCol.
            lblBg := dkCol.

            form2 isNil ifTrue:[
                lbl := drawableAction value:row
            ] ifFalse:[
                lbl := (description rendererType == #rowSelector) ifTrue:[form2] ifFalse:[nil]
            ]
        ] ifFalse:[
            (bgSelector notNil and:[(lblBg := row perform:bgSelector) notNil]) ifTrue:[
                dataSet paint:(lblBg on:(dataSet device)).
                dataSet fillRectangleX:0 y:y width:width height:innerH.                
            ] ifFalse:[
                lblBg := bg
            ].
            form2 isNil ifTrue:[
                (lbl := drawableAction value:row) notNil ifTrue:[
                    (fgSelector notNil and:[(lblFg := row perform:fgSelector) notNil]) ifFalse:[
                        lblFg := fg
                    ]
                ]
            ] ifFalse:[
                lbl := nil
            ]
        ].        
        lbl notNil ifTrue:[
            dataSet paint:lblFg on:lblBg.
            lbl displayOn:dataSet x:x y:y + textInset.
        ].
        y := y + rowHeight
    ].
    dataSet paint:fg on:bg.

    form notNil ifTrue:[
        y := yTop  + formInset.
        x := width - form width - sepSize.

        form2 isNil ifTrue:[
            nTimes timesRepeat:[
                dataSet displayForm:form x:x y:y.
                y := y + rowHeight.
            ]
        ] ifFalse:[
            x := x // 2.

            start to:stop do:[:i|
                (self at:i) == false ifTrue:[dataSet displayForm:form2 x:x y:y]
                                    ifFalse:[dataSet displayForm:form  x:x y:y].

                y := y + rowHeight.
            ]
        ].
    ].

    "/ ************************** DRAW SEPARATORS **************************

    drawColSep := description showColSeparator.
    w          := width - 1.

    sepSize == 1 ifTrue:[
        dataSet paint:fgColor.

        drawColSep ifTrue:[
            dataSet displayLineFromX:w y:yTop toX:w y:yBot
        ].
        description showRowSeparator ifTrue:[
            y := yTop - 1.

            nTimes timesRepeat:[
                y := y + rowHeight.
                dataSet displayLineFromX:0 y:y toX:w y:y.
            ]
        ].
        ^ self
    ].

    "/ draw 3D separators

    dkCol := dataSet separatorDarkColor.
    lgCol := dataSet separatorLightColor.

    drawColSep ifTrue:[
        dataSet paint:lgCol.
        y := yBot - 1. dataSet displayLineFromX:w y:yTop toX:w y:y.
        w := w    - 1. dataSet displayLineFromX:w y:yTop toX:w y:y.
        dataSet paint:dkCol.
        y := y - 1.
        w := w - 1. dataSet displayLineFromX:w y:yTop toX:w y:y.
        w := w - 1. dataSet displayLineFromX:w y:yTop toX:w y:y.
        w := width - 1.
        isLast := dataSet lastColumn == self.
    ].

    description showRowSeparator ifTrue:[
        drawColSep ifTrue:[
            n := w - 2
        ] ifFalse:[
            n := w.
        ].
        dataSet paint:lgCol.
        y := yTop - 1.

        nTimes timesRepeat:[
            y  := y + rowHeight.
            y1 := y - 1.
            dataSet displayLineFromX:0 y:y  toX:w y:y.
            dataSet displayLineFromX:0 y:y1 toX:w y:y1.
        ].

        dataSet paint:dkCol.
        y := yTop.

        nTimes timesRepeat:[
            y  := y + rowHeight.
            y1 := y - 4. dataSet displayLineFromX:0 y:y1 toX:n y:y1.
            y1 := y - 3. dataSet displayLineFromX:0 y:y1 toX:n y:y1.

            drawColSep ifTrue:[
                dataSet displayPointX:n y:y - 1.
                isLast ifFalse:[
                    dataSet displayPointX:w y:y1
                ]
            ]
        ].

        dataSet size == stop ifTrue:[
            dataSet paint:lgCol.
            dataSet displayPointX:n y:y - 1.
        ]
    ]

!

invalidate
    "set to invalidate; forces a recomputation
    "
    minWidth := width := nil.

! !

!DataSetColumn methodsFor:'editing'!

editorAt:aRowNr in:aView with:fg bg:bg

    ^ description editorOn:(dataSet at:aRowNr)
                     value:(self at:aRowNr)
                        in:aView
                      with:fg
                        bg:bg.


! !

!DataSetColumn methodsFor:'event handling'!

doesNotUnderstand:aMessage

    (description respondsTo:(aMessage selector)) ifTrue:[
        ^ aMessage sendTo:description
    ].
    ^ super doesNotUnderstand:aMessage

!

textInsetChanged:aTextInset
    "recompute all attributes
    "
    |lbl img|

    form2 isNil ifTrue:[                                    "/ has drawable labels
        textInset := dataSet verticalSpacing.

        dataSet size ~~ 0 ifTrue:[
            lbl := self drawableAt:1.

            (lbl isString or:[lbl isNumber]) ifTrue:[
                textInset := aTextInset
            ] ifFalse:[
                (lbl isNil and:[description printSelector isNil]) ifTrue:[
                    textInset := aTextInset.
                ]
            ]
        ]
    ].

    (img := form2 ? form) isNil ifTrue:[
        formInset := 0
    ] ifFalse:[        
        formInset := ( (dataSet rowHeight)
                      - img height
                      - (dataSet separatorSize)
                     ) // 2.

        form2 notNil ifTrue:[
            textInset := formInset
        ]        
    ].


! !

!DataSetColumn methodsFor:'grow & degrow'!

growWidth:n
    "grow the width for n pixels
    "
    width := width + n.


!

minWidth
    "get my minimum width required by the entries into the column
    "
    ^ minWidth notNil ifTrue:[minWidth] ifFalse:[self width]


!

setMinWidth
    "set the width to the required width. returns the psitive delta width
    "
    |deltaX|

    deltaX := width - minWidth.
    width  := minWidth.
  ^ deltaX.


! !

!DataSetColumn methodsFor:'initialization'!

on:aDSVColumnView description:aDescription columnNumber:aNumber
    "instance creation; set attributes dependent on the description
    "
    |rendererType device selector format idx type|

    columnNumber    := aNumber.
    dataSet         := aDSVColumnView.
    description     := aDescription.
    rendererType    := description rendererType.
    form            := width := form2 := nil.
    device          := dataSet device.
    label           := description rawLabel.
    drawableAction  := nil.
    fgSelector      := description foregroundSelector.
    bgSelector      := description backgroundSelector.
    backgroundColor := description backgroundColor.
    foregroundColor := description foregroundColor.

    backgroundColor notNil ifTrue:[
        backgroundColor := backgroundColor on:aDSVColumnView device
    ].
    foregroundColor notNil ifTrue:[
        foregroundColor := foregroundColor on:aDSVColumnView device
    ].

    label isImage ifTrue:[
        label := label onDevice:device
    ].

    rendererType == #CheckToggle ifTrue:[
        form  := dataSet toggleOnButton.
        form2 := dataSet toggleOffButton.
        ^ self
    ].

    rendererType == #rowSelector ifTrue:[
        form2 := dataSet rowSelectorButton.
        ^ self
    ].

    (rendererType == #ComboBox or:[rendererType == #ComboList]) ifTrue:[
        form := dataSet comboViewButton.
    ].

    selector := description printSelector.

    selector notNil ifTrue:[
        drawableAction := [:aRow| aRow perform:selector with:dataSet ].
        ^ self
    ].

    (     (format := description formatString) notNil
     and:[(type   := description type) == #number or:[type == #numberOrNil]]
    ) ifTrue:[
        "/ has a format string for number (supports only floats)

        (idx := format indexOf:$.) ~~ 0 ifTrue:[
            idx := format size - idx
        ].
        format := '%0.', idx printString, 'f'.

        drawableAction := [:aRow||n|
            n := description row:aRow at:columnNumber.
            n isReal ifTrue:[n := n asFloat printfPrintString:format].
            n
        ]
    ] ifFalse:[         "/ default: no format string
        drawableAction := [:aRow| description row:aRow at:columnNumber ]
    ]


! !

!DataSetColumn methodsFor:'private'!

drawableAt:aRowNr
    "get the drawable label at an index
    "
    form2 isNil ifTrue:[
        ^ drawableAction value:(dataSet at:aRowNr)
    ].
    ^ nil
! !

!DataSetColumn methodsFor:'queries'!

canSelect:aRowNr
    "returns true if cell in column is selectable
    "
    |s|

    description canSelect ifTrue:[
        ^ ((s := description selectSelector) isNil or:[(dataSet at:aRowNr) perform:s])
    ].
    ^ false

!

heightOfHighestRow
    "returns the height of the highest row in pixels
    "
    |hObj hMin end|

    hMin := (form ? dataSet font) heightOn:dataSet device.

    (hObj := description height) == 0 ifTrue:[
        form2 isNil ifTrue:[                                    "/ has drawable labels
            "/ search first none empty drawable object
            end := 10 min:dataSet size.

            1 to:end do:[:aRowNr| |lbl|
                lbl := self drawableAt:aRowNr.

                lbl notNil ifTrue:[
                    (lbl respondsTo:#heightOn:) ifTrue:[
                        hObj := lbl heightOn:dataSet
                    ] ifFalse:[
                        hObj := lbl displayString heightOn:dataSet
                    ].
                    ^ hMin max:hObj
                ]
            ]
        ]
    ].
    ^ hMin max:hObj.
! !

!DataSetColumn methodsFor:'searching'!

findRowNrStartingWithChar:aChar start:start stop:stop
    "find the first row starting at start to stop, which drawable label
     starts with the character, aChar. The index of the detected row is
     returned or if no row is found 0.
    "
    |char|

    form2 isNil ifTrue:[                        "/ has drawable labels
        char  := aChar asLowercase.

        start to:stop do:[:aRowNr| |lbl|
            lbl := self drawableAt:aRowNr.

            lbl isString ifTrue:[
                (lbl size ~~ 0 and:[(lbl at:1) asLowercase == char]) ifTrue:[
                    ^ aRowNr
                ]
            ] ifFalse:[
                lbl isNil ifFalse:[
                    ^ 0
                ]
            ]
        ]
    ].
    ^ 0

! !

!DataSetColumn class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libwidg2/DataSetColumn.st,v 1.11 1997-11-14 09:26:21 ca Exp $'
! !
